require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xblautobitlocker',
  dscmeta_resource_friendly_name: 'xBLAutoBitlocker',
  dscmeta_resource_name: 'MSFT_xBLAutoBitlocker',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xBitlocker',
  dscmeta_module_version: '1.4.0.0',
  docs: 'The DSC xBLAutoBitlocker resource type.
         Automatically generated from version 1.4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_recoverykeypath: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_hardwareencryption: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_drivetype: {
      type: "Enum['Fixed', 'Removable']",
      desc: 'The type of volume, as reported by Get-Volume, to auto apply Bitlocker to',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_encryptionmethod: {
      type: "Optional[Enum['Aes128', 'Aes256']]",
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_startupkeyprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_passwordprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_autounlock: {
      type: 'Optional[Boolean]',
      desc: 'Whether volumes should be enabled for auto unlock using Enable-BitlockerAutoUnlock',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_adaccountorgroup: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_service: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_password: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: "NOTE: Username doesn't matter for the credential. Just put the Password in the Password field",
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_adaccountorgroupprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_recoverykeyprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_primaryprotector: {
      type: "Enum['PasswordProtector', 'RecoveryPasswordProtector', 'StartupKeyProtector', 'TpmProtector']",
      desc: ' ',

      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_recoverypasswordprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_usedspaceonly: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_skiphardwaretest: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_pin: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: "NOTE: Username doesn't matter for the credential. Just put the Pin in the Password field",
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_mindiskcapacitygb: {
      type: 'Optional[Integer[-2147483648, 2147483647]]',
      desc: 'If specified, only disks this size or greater will auto apply Bitlocker',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'SInt32',
      mof_is_embedded: false,
    },
    dsc_startupkeypath: {
      type: 'Optional[String]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'String',
      mof_is_embedded: false,
    },
    dsc_tpmprotector: {
      type: 'Optional[Boolean]',
      desc: ' ',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
  },
)
