require 'puppet/resource_api'

Puppet::ResourceApi.register_type(
  name: 'dsc_xbltpm',
  dscmeta_resource_friendly_name: 'xBLTpm',
  dscmeta_resource_name: 'MSFT_xBLTpm',
  dscmeta_resource_implementation: 'MOF',
  dscmeta_module_name: 'xBitlocker',
  dscmeta_module_version: '1.4.0.0',
  docs: 'The DSC xBLTpm resource type.
         Automatically generated from version 1.4.0.0',
  features: ['simple_get_filter', 'canonicalize', 'custom_insync'],
  attributes: {
    name: {
      type:      'String',
      desc:      'Description of the purpose for this resource declaration.',
      behaviour: :namevar,
    },
    validation_mode: {
      type:      'Enum[property, resource]',
      desc:      'Whether to check if the resource is in the desired state by property (default) or using Invoke-DscResource in Test mode (resource).',
      behaviour: :parameter,
      default:   'property',
    },
    dsc_allowphysicalpresence: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that the provisioning process may send physical presence commands that require a user to be present in order to continue.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowimmediatereboot: {
      type: 'Optional[Boolean]',
      desc: 'Whether the computer can rebooted immediately after initializing the TPM.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_allowclear: {
      type: 'Optional[Boolean]',
      desc: 'Indicates that the provisioning process clears the TPM, if necessary, to move the TPM closer to complying with Windows Server 2012 standards.',

      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'Boolean',
      mof_is_embedded: false,
    },
    dsc_psdscrunascredential: {
      type: 'Optional[Struct[{ user => String[1], password => Sensitive[String[1]] }]]',
      desc: ' ',
      behaviour: :parameter,
      mandatory_for_get: false,
      mandatory_for_set: false,
      mof_type: 'PSCredential',
      mof_is_embedded: true,
    },
    dsc_identity: {
      type: 'String',
      desc: 'A required string value which is used as a Key for the resource. The value does not matter, as long as its not empty.',
      behaviour: :namevar,
      mandatory_for_get: true,
      mandatory_for_set: true,
      mof_type: 'String',
      mof_is_embedded: false,
    },
  },
)
